---------------------------------
-- Configuration Section start --
---------------------------------

-- flash the window on incoming whispers
local FlashOnWhisper = true

-- flash the window when your toon enters combat
local FlashOnEnterCombat = false

-- flash when the role select window in a raid pops up
local FlashOnRoleSelect = false

-- flash when a pvp invite is pending
local FlashOnPvpInvite = false

-- flash when an lfg invite is pending
local FlashOnLfgInvite = false

-- flash when a ready check was issued
local FlashOnReadyCheck = false

-- allow flashing from unknown sources (i.e. addons)
local FlashFromUnknownSource = true

-- shows debug messages in chat
local EnableDebugMessage = true

---------------------------------
-- Configuration Section end   --
---------------------------------

-- Logic code start

local flashIconCallerList =
{
    SetInCombat = FlashOnEnterCombat,
    ChatFrame_MessageEventHandler = FlashOnWhisper,
    RolePollPopup_Show = FlashOnRoleSelect,
    PVPFramePopup_SetupPopUp = FlashOnPvpInvite,
    LFGEventFrame_OnEvent = FlashOnLfgInvite,
    LFGListInviteDialog_Show = FlashOnLfgInvite,
    ShowReadyCheck = FlashOnReadyCheck
}

-- Save the reference to the original function, then replace the 
_G['FlashClient_ORIG'] = FlashClientIcon
FlashClientIcon = function()
    -- get some caller data like file, line and function name
    local file, line, func = string.match(debugstack(2, 1, 0), "([^:]+):([0-9]+): in function ([^\n]+)")

    if(file == nil) then
        print("Someone unexpected called FlashClientIcon! Please report this to the developer!")
        print(debugstack())
        if(FlashFromUnknownSource) then
            FlashClient_ORIG();
        end
        return
    end

    -- functions inside XMLs dont have a filename nor a useful function name
    -- those functions look like [string ":OnShow"] - i just check for the leading bracket
    if(func:sub(2,2) == "[") then
        -- function is defined inside a xml, need to walk the stack :/
        for i in string.gmatch(debugstack(2), "(.-)\r?\n") do
            file, line, func = string.match(i, "([^:]+):([0-9]+): in function ([^\n]+)")
            if(file ~= nil and file:sub(1, 9) == "Interface") then
                -- we found a function which is defined in a file
                -- we just use this one and hope for the best :P
                break
            end
        end
    end
    -- remove the leading and trailing quotes from the func name
    func = func:sub(2, func:len()-1) 
    
    -- check if the caller is blacklisted from using the FlashClientIcon function
    local state = flashIconCallerList[func]
    if(state == false) then
        if(EnableDebugMessage) then
            print(func.." is blacklisted")
        end
    elseif (not FlashFromUnknownSource and state == nil) then
        if(EnableDebugMessage) then
            print(func.." is blacklisted (unknown function)")
        end
    else
        if(EnableDebugMessage) then
            print(func.." is not blacklisted")
        end
        FlashClient_ORIG();
    end
end
